@@ -1,5 +1,5 @@
 /*
-#define RCS_ID $Id: pb.c,v 1.22 2001/03/16 08:45:37 oz6bl Exp $
+#define RCS_ID $Id: pb.c,v 1.33 2008/01/05 16:20:22 oz6bl Exp $
  *   Copyright 1992, 1993, 1994 John Melton (G0ORX/N6LYT)
  *              All Rights Reserved
  *
@@ -23,7 +23,6 @@
  *	Uses ncurses to handle display control
  *	Automatic update of directory
  *	Database of files to be downloaded
- *	Extended control of pb by UDP packets
  *	Automatic selection of files to download based on a rule file 
  *   
  */
@@ -127,11 +126,50 @@
 
 	-	Changed the pb->pbctrl communication to also use an XML syntax
 	-	Made the control FIFO fixed: it is always /etc/pbpg/pb-fifo
+
+	V2.01
+
+	-	Added a filter to ProcessFrame so that only frames from the
+		satellite of interest are processed
+	-	Fixed a cosmetic error in the name of the KISS files and in the
+		time stamp in pb.log (Tnx Steve, VK5ASF)
+
+	V2.02
+
+	-	Changed some include file locations
+	-	Changed the information in the 'download' window to display 
+		the percentage of the file received, provided the file header
+		has been received
+	-	Changed the struct for 'src' and 'dest' from sockaddr_ax25 to
+		full_sockaddr_ax25, thus suppressing an annoying system log
+		message about use of obsolete structures
+
+	V2.10
+
+	-	Removed the usage of an UDP port for control of PB
+	-	Minor corrections
+	-	Added #include <netax25/axli.h> to remove a nasty warning
+		and #include <netrose/rose.h> to facilitate compilation 
+		(thanks Steve, VK5ASF)
+
+	V2.20
+
+	-	Made the comparisons for possible download case insensitive
+	-	Added a check so that downloaded files, whose size has changed
+       		since last download, will be downloaded again 
+	-	Suppressed the 'unknown tag' error message by suppressing 
+		sending of empty LOG and FILES messages.
+	-	Changed some sprintf to snprintf (thanks John, EI7IG)
+	-	Moved the filter in ProcessFrame so that all frames received 
+		are logged in the KISS file 
+	-	Skip the newly introduced parameter PREDICT_SERVER in pb.conf
+       		when scanning for parameters in the configuration file. 
+
 */
 
-#define VERSION_STRING "version 2.00 by oz2aba and oz6bl"
+#define VERSION_STRING "version 2.20 by oz2aba and oz6bl"
 #define USAGE_STRING "Usage: pb [-s satellite] [-m mycall] [-n satname] \n\t\
-[-M maxdays] [-U UDPport] [-k kissfile] [-v]"
+[-M maxdays] [-k kissfile] [-v]"
 
 #include <sys/time.h>
 #include <sys/types.h>
@@ -140,8 +178,6 @@
 
 #include <netinet/in.h>
 #include <linux/if_ether.h>
-#include <linux/ax25.h>
-#include <linux/rose.h>
 
 #include <fcntl.h>
 #include <unistd.h>
@@ -160,7 +196,8 @@
 #include "crc.h"
 #include "fifo.h"
 
-#include <netax25/axconfig.h>
+#include <netax25/ax25.h>
+#include <netrose/rose.h>
 #include <netax25/axlib.h>
 
 #define REQUEST_NONE 0
@@ -177,7 +214,6 @@ void writelog (int, char *) ;
 void UpdateStatus(void) ;
 void FillDirectory(void) ;
 void FillFile(int *) ;
-void CancelFill(void) ;
 int ThisFileShouldBeDownloaded(HEADER *) ;
 void InitializeDownload(void) ;
 void ToMonitor (char *, char *) ;
@@ -192,9 +228,8 @@ int debugmask = 0 ;
 /* Program parameters as read from pb.conf or the command line*/
 char Satellite	[100] ;
 char SatName	[100] ;
-char MyCall		[100] ;
+char MyCall	[100] ;
 char MaxDays	[100] ;
-char UDPPort	[100] ;
 char KissLog	[100] ;
 
 
@@ -202,7 +237,6 @@ char KissLog	[100] ;
 
 char MyPBCall    [100];
 int  MyPBCallLength ;
-unsigned int UdpPort     = 5100;
 int	     kisslog     = 0;
 unsigned long maxDays = 5;
 
@@ -220,11 +254,8 @@ unsigned long int lastByteWrite = 0;
 int s_input;
 int s_file;
 int s_directory;
-struct sockaddr_ax25	src;
-struct sockaddr_ax25	dest;
-
-struct sockaddr_in request_addr;
-int s_request;		/* Handle for receiving internal requests */
+struct full_sockaddr_ax25	src;
+struct full_sockaddr_ax25	dest;
 
 #define	TIMEOUT		2
 #define TIMEOUTFLUSH	30
@@ -272,6 +303,24 @@ void signal_reinit (int s) {
 	signal(SIGHUP, signal_reinit) ;
 	InitializeDownload() ;
 }
+
+int percentage (int f) {
+	int fileSize, totalHoles;
+	HOLE *hole;
+
+	// See if a pfh header has been received
+	if (of[f].hdrSeen == 0) return -1 ; // exit if not
+
+	// Get file size 
+	fileSize = of[f].fileSize;
+	totalHoles = 0;
+	hole = of[f].firstHole;
+	while (hole != NULL) {
+		totalHoles += hole->finish - hole->start;
+		hole = hole->nextHole;
+	}
+	return (100 * (fileSize - totalHoles)) / fileSize;
+}
 /*
  *	Signal handler to handle the SIGTERM signal
  */
@@ -335,11 +384,11 @@ void ToMonitor (char *tag, char *text) {
 		mon_write("ERROR - text too long") ;
 	else {
 		if (textlen == 0)
-			sprintf (buffer, "<%s/>", tag) ;
+			snprintf (buffer, sizeof(buffer), "<%s/>", tag) ;
 		else
-			sprintf (buffer, "<%s>%s</%s>", tag, text, tag) ;
+			snprintf (buffer, sizeof(buffer), "<%s>%s</%s>", tag, text, tag) ;
 		if ((debugmask & DEBUG_TO_FIFO) == DEBUG_TO_FIFO) {
-			sprintf (debugline, "To FIFO  :%s", buffer) ; 
+			snprintf (debugline, sizeof(debugline), "To FIFO  :%s", buffer) ;
 			WriteToLog (debugline) ; }
 		mon_write (buffer) ; 
 	}
@@ -354,9 +403,11 @@ void writelog (int scrno, char *text)
 	/* Remove the tail if it is a \n */
 	while (text [strlen (text)-1] == '\n' || text [strlen (text)-1] == ' ')
 		text [strlen (text) - 1] = '\0';
-	encodePCData (text, temp) ;
-	if (scrno == LOG) ToMonitor("LOG", temp) ;
-	else ToMonitor("FILES", temp);
+	if (strlen(text) > 0) { // Don't send empty messages
+		encodePCData (text, temp) ;
+		if (scrno == LOG) ToMonitor("LOG", temp) ;
+		else ToMonitor("FILES", temp);
+	}
 }
 
 /*
@@ -847,9 +898,12 @@ void LoadFile(int fileId, int f)
 		}
 	}
 	/* now update the download window */
-		sprintf (txt, "%4x  %4d  %6d  %s",
-		              of[f].fileId, of[f].holes, 0,
-		              (of[f].hdrSeen == 1 ? "Y" : "N"));
+		if (of[f].hdrSeen == 0)
+			snprintf (txt,sizeof(txt), "%4x  %4d  %6d  %s",
+				of[f].fileId, of[f].holes, 0, "N ");
+		else  
+			snprintf (txt, sizeof(txt), "%4x  %4d  %6d  %2d",
+				of[f].fileId, of[f].holes, 0, percentage(f));
 		writedownload (f + 1, txt);
 
 }
@@ -1055,9 +1109,15 @@ void BroadcastFile(unsigned char *buffer, int length)
 		fwrite(data, 1, dataLength, of[i].file);
 
 		/* Update the download window */
-		sprintf (txt, "%4x  %4d  %6d  %s",
-		              of[i].fileId, of[i].holes, dataOffset,
-		              (of[i].hdrSeen == 1 ? "Y" : "N"));
+		if (of[i].hdrSeen == 0) {
+			sprintf (txt, "%4x  %4d  %6d  %s",
+		              of[i].fileId, of[i].holes, dataOffset, "N ");}
+		else {
+			sprintf(fileName, "%x.hol", fh->fileId);
+			sprintf (txt, "%4x  %4d  %6d  %2d",
+		        	of[i].fileId, of[i].holes, dataOffset,
+				percentage(i));
+		}
 		writedownload (i + 1, txt);
 
 		/* see if it has all been downloaded */
@@ -1244,37 +1304,8 @@ void ProcessFrame(void)
 	char viaCall[10];
 	char *s;
 
-	/* Check for runts/pygmygs */
-	if (bufSize < 15) {
-		char	st[10];
-		int	x;
-		sprintf (writebuf, "Runt:");
-		for (x = 0; x < bufSize; x++) {
-			sprintf (st, " %02x", buf [x]);
-			strcat (writebuf, st);
-		}
-		writelog (LOG, writebuf);
-		return;
-	}
-	
-	bytes += bufSize - 1 ;
-
-	/* decode the to/from address */
-	/* dont expect via address, but saves last if any */
-	strcpy(toCall, ax25_ntoa((const ax25_address *)(buf+n)));
-
-	//via = ConvertCall(buf + n, toCall);
-	n += 7;
-
-	via = ConvertCall(buf + n, fromCall);
-	n += 7;
-
-	while (via)
-	{
-		via = ConvertCall(buf + n, viaCall);
-		n += 7;
-	}
-
+	// Log all frames received on the interface -
+	// even if they are not from 'our' satellite
 	if (fpkiss != NULL)
 	{
 		fputc(0300, fpkiss);		/* FEND */
@@ -1300,7 +1331,41 @@ void ProcessFrame(void)
 
 		fputc(0300, fpkiss);		/* FEND */
 	}
+	
+	/* Check for runts/pygmygs */
+	if (bufSize < 15) {
+		char	st[10];
+		int	x;
+		sprintf (writebuf, "Runt:");
+		for (x = 0; x < bufSize; x++) {
+			sprintf (st, " %02x", buf [x]);
+			strcat (writebuf, st);
+		}
+		writelog (LOG, writebuf);
+		return;
+	}
+	
+	bytes += bufSize - 1 ;
 
+	/* decode the to/from address */
+	/* dont expect via address, but saves last if any */
+	strcpy(toCall, ax25_ntoa((const ax25_address *)(buf+n)));
+	i = strlen (toCall) ;
+	/* Remove a '-0' SSID from toCall */
+	if ((toCall[i-2] == '-') && (toCall[i-1] == '0')) toCall[i-2] = '\0';
+	n += 7;
+
+	via = ConvertCall(buf + n, fromCall);
+	n += 7;
+
+	while (via)
+	{
+		via = ConvertCall(buf + n, viaCall);
+		n += 7;
+	}
+
+	// see if the frame is from the satellite of interest
+	if (ax25_cmp ((ax25_address *)(buf+8), (ax25_address *)&dest.fsa_ax25.sax25_call)== 1) return ;
 
 	/* check for a UI frame */
 	if ((buf[n] & 0xEF) != 0003)
@@ -1378,12 +1443,12 @@ void ProcessFrame(void)
 		sprintf (writebuf, "%s", buf + n);
 		notprintable (writebuf);
 
-		if ((ptr = strstr (writebuf, "B: ")) || 
-		    (ptr = strstr (writebuf, "b:"))) {
+		if ((ptr = strstr (writebuf, "B: ")) ||
+		    (ptr = strstr (writebuf, "B:"))) {
 			/* ByteCounter message */
 			curByteRead  = dirBytes + fileBytes ;
-			curByteWrite = atol (ptr + 4) ; 
-				/* 4 is cheating for the 9600 Bd satellites */
+			curByteWrite = atol (ptr + 2) ; 
+				/* Start right after the ':' */
 			if (lastByteRead == 0) {
 				writelog (LOG, writebuf);
 			} else {
@@ -1391,11 +1456,7 @@ void ProcessFrame(void)
 				l1 = (curByteRead  - lastByteRead);
 				l2 = (curByteWrite - lastByteWrite);
 				if (l2 != 0L) l3 = (100L * l1) / l2;
-				sprintf (writebuf, "%s    (%ld / %ld = %ld%%)",
-				         buf + n,
-				         (curByteRead - lastByteRead),
-				         (curByteWrite - lastByteWrite),
-				         l3);
+				sprintf (writebuf, "%s    (%ld / %ld = %ld%%)", buf + n, l1, l2, l3);
 				notprintable (writebuf);
 				writelog (LOG, writebuf);
 			}
@@ -1488,51 +1549,6 @@ void TimeoutCb(void)
 		SaveHoleList(MAXFILES, "pfhdir.hol");
 }
 
-/*
- *	callback function when a another app calls us
- */
-void CommandRequest(void)
-{
-	struct request request;
-
-	if (recv(s_request, (char *)&request, sizeof(struct request), 0) == sizeof(struct request))
-	{
-		switch (request.type)
-		{
-			case REQ_TYPE_CANCEL:
-				CancelFill();
-				break;
-			case REQ_TYPE_DIR:
-/*				FillDirectory();	*/
-				break;
-			case REQ_TYPE_FILE:
-/*				sprintf(id, "%lx", request.fileId);   */
-/*				id = request.fileId;	*/
-/*				FillFile(0);	*/
-				break;
-			case REQ_TYPE_REFILE:
-				writelog (FILES, "download.req updated");
-				/* If requester is idle check it */
-/* ???? This hack is disabled.The request will be sent on next PB frame
-				if (request_no == REQUEST_NONE)
-					UpdateRequest (REQUEST_NONE);
-*/
-				break;
-			case REQ_TYPE_REINIT:
-				InitializeDownload() ;
-				break ;
-			case REQ_TYPE_QUIT:
-				writelog (FILES, "Stop request received");
-				stopFlag = 1;
-				break;
-		}
-	}
-	else
-	{
-		perror("recv - FillRequest");
-	}
-}
-
 /*
  *	the user wants to exit this program
  */
@@ -1565,8 +1581,8 @@ void QuitCb(void)
 				ave_rate = bytes / (end_time - start_time);
 
 			tm = gmtime(&start_time);
-			fprintf(fp, "%02d/%02d/%02d %02d:%02d:%02d - ",
-				tm->tm_mday, tm->tm_mon + 1, tm->tm_year,
+			fprintf(fp, "%02d/%02d/%04d %02d:%02d:%02d - ",
+				tm->tm_mday, tm->tm_mon + 1, tm->tm_year+1900,
 				tm->tm_hour, tm->tm_min, tm->tm_sec);
 
 			tm = gmtime(&end_time);
@@ -1719,31 +1735,6 @@ void FillFile(int *reqlen)
 	UpdateStatus () ;
 }
 
-
-
-/*
- *	Cancel a Fill request		????
- */
-void CancelFill(void)
-{
-#ifdef undefined
-	switch (request_state)
-	{
-		case REQUEST_DIR:
-			writelog (FILES, "Cancel Fill: directory");
-			break;
-		case REQUEST_FILE:
-			sprintf  (writebuf, "Cancel Fill: file %lx", id);
-			writelog (FILES, writebuf);
-			break;
-		default:
-			break;
-	}
-
-	request_state = REQUEST_NONE;
-#endif
-}
-
 void output_init(void) {
 	char txt[100] ;
 	
@@ -1834,7 +1825,6 @@ int main(int argc, char **argv)
 	int n, fifo_fd;
 	char tmpMyCall	[100] ;
 	char tmpMaxDays	[100] ;
-	char tmpUDPPort	[100] ;
 	char tmpKissLog	[100] ;
 
 	fd_set		readfds;
@@ -1846,7 +1836,7 @@ int main(int argc, char **argv)
 	char		*s, s1[100], s2[100];
 
 	/* Get command line options but do not check them yet */
-	while ((n = getopt(argc, argv, "s:m:n:M:U:k:v")) != -1) {
+	while ((n = getopt(argc, argv, "s:m:n:M:k:v")) != -1) {
 		switch (n) {
 			case 's':
 				strcpy(Satellite, optarg);
@@ -1860,9 +1850,6 @@ int main(int argc, char **argv)
 			case 'M':
 		      	strcpy(MaxDays, optarg);
 				break;
-			case 'U':
-				strcpy(UDPPort, optarg);
-		        break;
 			case 'k':
 				strcpy(KissLog, optarg);
 		        break;
@@ -1907,12 +1894,11 @@ int main(int argc, char **argv)
 			if (!among || within) strcpy (tmpMyCall, s2); continue;}
 		if (! strcmp (s1, "MAXDAYS")) {
 			if (!among || within) strcpy (tmpMaxDays, s2); continue;}
-		if (! strcmp (s1, "UDPPORT")) {
-			if (!among || within) strcpy (tmpUDPPort, s2); continue;}
 		if (! strcmp (s1, "KISSLOG")) {
 			if (!among || within) strcpy (tmpKissLog, s2); continue;}
 		if (! strcmp (s1, "MAXIDLE")) {continue; /* PG parameter - ignore */}
 		if (! strcmp (s1, "MAXWAIT")) {continue; /* PG parameter - ignore */}
+		if (! strcmp (s1, "PREDICT_SERVER")) {continue; /* PBctl parameter - ignore */}
 		fprintf (stderr, "Error in config file line <%d>\n", n);
 	}
 	if (!within) {
@@ -1922,7 +1908,6 @@ int main(int argc, char **argv)
 
 	/* Copy config. file values if not defined on the command line */
 	if (MyCall[0]  == '\0') strcpy (MyCall,  tmpMyCall) ;
-	if (UDPPort[0] == '\0') strcpy (UDPPort, tmpUDPPort) ;
 	if (MaxDays[0] == '\0') strcpy (MaxDays, tmpMaxDays) ;
 	if (KissLog[0] == '\0') strcpy (KissLog, tmpKissLog) ;
 	
@@ -1937,15 +1922,11 @@ int main(int argc, char **argv)
 		maxDays = atoi(MaxDays);
 		if (maxDays <= 0) maxDays = 5;
 	}
-	if (UDPPort [0] != '\0') {
-		UdpPort = atoi (UDPPort);
-		if (UdpPort < 1024) UdpPort = 5100;
-	}
 
 	/* Create the packet addresses to network format */
 	strcat(Satellite, "-11");
-	ax25_aton(MyCall, (struct full_sockaddr_ax25 *)&src);
-	ax25_aton(Satellite, (struct full_sockaddr_ax25 *)&dest);
+	ax25_aton(MyCall, &src);
+	ax25_aton(Satellite, &dest);
 	strcpy (writebuf, MyCall) ;
 	if ((s = strtok(writebuf, "-")) != NULL)
 		strcpy (MyPBCall, s) ;
@@ -1982,7 +1963,7 @@ int main(int argc, char **argv)
                 return 1 ; }
 
         /* open up the AX25 datagram socket to send file requests */
-        if ((s_file = socket(AF_AX25, SOCK_RAW, PID_FILE)) == -1)
+        if ((s_file = socket(PF_AX25, SOCK_RAW, PID_FILE)) == -1)
         {
                 perror("socket");
                 return(1);
@@ -1995,7 +1976,7 @@ int main(int argc, char **argv)
         }
 
         /* open up the AX25 datagram socket to send directory requests */
-        if ((s_directory = socket(AF_AX25, SOCK_RAW, PID_DIRECTORY)) == -1)
+        if ((s_directory = socket(PF_AX25, SOCK_RAW, PID_DIRECTORY)) == -1)
         {
                 perror("socket");
                 return(1);
@@ -2007,29 +1988,13 @@ int main(int argc, char **argv)
                 return(1);
         }
 
-	/* open up a udp socket to receive file fill requests from other apps */
-	memset((char *)&request_addr, 0, sizeof(request_addr));
-	request_addr.sin_family      = AF_INET;
-	request_addr.sin_addr.s_addr = INADDR_ANY;
-	request_addr.sin_port        = htons(UdpPort);
-
-	if ((s_request = socket(AF_INET, SOCK_DGRAM, 0)) == -1)
-	{
-		perror("socket - s_request");
-	}
-		
-	if (bind(s_request, (struct sockaddr *)&request_addr, sizeof(request_addr)) == -1)
-	{
-		perror("bind - s_request");
-	}
-
 	/* Open the KissLog file */
 	if (KissLog [0] != 0) {
 		time_t timenow ;
 		struct tm *tm ;
 		time (&timenow) ;
 		tm = gmtime (&timenow) ;
-		sprintf (KissLog, "%02d%02d%02d%02d.kss", tm->tm_year, tm->tm_mon+1, tm->tm_mday, tm->tm_hour) ;
+		sprintf (KissLog, "%04d%02d%02d%02d.kss", tm->tm_year+1900, tm->tm_mon+1, tm->tm_mday, tm->tm_hour) ;
 		if ((fpkiss = fopen(KissLog, "ab")) == NULL)  {
 			perror ("Error opening the KissLog file");
 		}
@@ -2075,7 +2040,7 @@ int main(int argc, char **argv)
 	/* Close stdin, stdout, and stderr */
 	//for (n = 0 ; n < 3 ; n++) close (n) ;
 		
-	/* Main loop. Start select on DataInput, UDPInput, FIFO and timeout */
+	/* Main loop. Start select on DataInput, FIFO and timeout */
 	timeout.tv_sec  = TIMEOUT;
 	timeout.tv_usec = 0;
 	timeoutflush    = TIMEOUTFLUSH + time (0);
@@ -2084,7 +2049,6 @@ int main(int argc, char **argv)
 		FD_ZERO (&readfds);
 		if (fifo_fd != -1) FD_SET(fifo_fd, &readfds) ;
 		FD_SET (s_input,        &readfds);
-		FD_SET (s_request,      &readfds);
 		n = select (32, &readfds, NULL, NULL, &timeout);
 		if (n < 0) {
 			if (errno == EINTR) continue ; /* restart the operation */
@@ -2109,7 +2073,6 @@ int main(int argc, char **argv)
 		if ((fifo_fd != -1) && (FD_ISSET (fifo_fd, &readfds))) {
 			/* Input from FIFO */
 			ctrl_read (fifo_pipe) ; }
-		if (FD_ISSET (s_request, &readfds)) CommandRequest ();
 	}
 
 	/* Terminate the program */
